#
#  MainController.rb
#  iTunes2PSP
#
require 'osx/cocoa'
require 'osx/aeosa'

class MainController < OSX::NSObject
	include OSX

	ib_outlets :playlists_view, :sync_psp_button, :total_information

	def initialize
		@psp_path = "Untitled:PSP:MUSIC"

		result = OSX.do_osascript %{
			----------------------------------------- iTunes$B$+$i%W%l%$%j%9%HCj=P(B
			tell application "iTunes"
				set my_playlists to {}
				repeat with current_playlist in user playlists
					set my_tracks to {}
					set n to name of current_playlist
					copy n to the end of my_tracks
					set n to count of tracks of current_playlist
					copy n to the end of my_tracks
					set n to size of current_playlist
					copy n to the end of my_tracks
					set n to duration of current_playlist
					copy n to the end of my_tracks
					copy my_tracks to the end of my_playlists
				end repeat
				my_playlists
			end tell
		}

		identifiers = [
			["name", :to_s],
			["track_number", :to_i],
			["raw_size", :to_i],
			["raw_time", :to_i],
		]
		index = 1
		@playlists = Array.new
		result.map.each do |playlist|
			@track_info = Hash.new
			@track_info["check"] = 0
			@track_info["index"] = index
			index += 1
			i = 0
			playlist.map.each do |track|
				@track_info[identifiers[i][0]] =
					track.to_rbobj.send(identifiers[i][1])
				i += 1
			end

			s = @track_info["raw_size"]
			@track_info["size"] = sprintf "%dMB", s / 1024 / 1024

			s = @track_info["raw_time"]
			@track_info["time"] = sprintf "%d:%02d", s / 60, s % 60

			@playlists.push @track_info
		end
	end

	def awakeFromNib
	end

	# data sourse
	def numberOfRowsInTableView(tblView)
		@playlists.size
	end

	def tableView_objectValueForTableColumn_row(tblView, col, row)
		@playlists[row][col.identifier.to_s]
	end

	# delegate
	def tableViewSelectionDidChange(notification)
		total_num = 0
		total_size = 0
		total_time = 0
		iter = @playlists_view.selectedRowEnumerator
		while index = iter.nextObject do
			playlist = @playlists[index.to_i]
			total_num += playlist["track_number"]
			total_size += playlist["raw_size"]
			total_time += playlist["raw_time"]
		end
		total_size = sprintf "%dMB", total_size / 1024 / 1024
		total_time = sprintf "%d:%02d", total_time / 60, total_time % 60
		@total_information.setStringValue(
			"#{total_num}  #{total_size}  #{total_time}")
	end

	# action
	def open_psp(sender = nil)
		result = OSX.do_osascript %{
			tell application "Finder"
				try
					open ("#{@psp_path}" as alias)
				on error
					display dialog "PSP not found" \
						with icon caution buttons {"OK"}
				end try
			end tell
		}
	end

	def sync_psp(sender = nil)
		copylist = ""
		iter = @playlists_view.selectedRowEnumerator
		while index = iter.nextObject do
			playlist = @playlists[index.to_i]
			copylist << "," if copylist != ""
			copylist << playlist["index"].to_s
		end
		return if copylist == ""

		result = OSX.do_osascript %{
			try
				("#{@psp_path}" as alias)
				set copylist to {#{copylist}}
			on error
				display dialog "PSP not found" with icon caution buttons {"OK"}
				set copylist to {}
				return ""
			end try

			repeat with index in copylist
				-------- iTunes $B$+$i%3%T!<$9$k%W%l%$%j%9%HL>$H%U%!%$%k%j%9%HCj=P(B
				tell application "iTunes"
					set current_playlist to user playlist index
					set playlist_name to name of current_playlist
					set playlist_files to {}
					repeat with current_track in file tracks of current_playlist
						set file_location to location of current_track
						copy file_location to the end of playlist_files
					end repeat
				end tell

				tell application "Finder"
					--------------------------------------- $B%U%!%$%kB8:_%A%'%C%/(B
					set folder_name to "#{@psp_path}:" & playlist_name
					set playlist to ""
					set copyfiles to {}
					repeat with my_file in playlist_files
						set filename to name of my_file
						if filename ends with ".mp3" then
							set playlist to playlist & filename & "\n"
							try
								(folder_name & ":" & filename) as alias
							on error
								copy my_file to the end of copyfiles
							end try
						end if
					end repeat

					if (count of playlist) /= 0 then
						-------------------- PSP$B>e$K%W%l%$%j%9%HL>$G%U%)%k%@:n@.(B
						try
							folder_name as alias
						on error
							try
								make new folder at "#{@psp_path}" \
									with properties {name:playlist_name}
							on error
								display dialog "make #{@psp_path}:" & \
									playlist_name & " failed" \
										with icon caution buttons {"OK"}
								return
							end try
						end try
					end if

					if (count of copyfiles) /= 0 then
						----------------------------------------- $B%U%!%$%k%3%T!<(B
						try
							duplicate copyfiles to (folder_name as alias)
						on error
							display dialog "duplicate to " & folder_name & \
									" failed" with icon caution buttons {"OK"}
							return
						end try

						---------------------------------------- m3u$B%U%!%$%k:n@.(B
						set m3ulist to \
							folder_name & ":" & playlist_name & ".m3u"
						try
							(m3ulist as alias)
						on error
							try
								make new file at (folder_name as alias) \
									with properties \
										{name:playlist_name & ".m3u"}
							on error
								display dialog "make " & \
									folder_name & ":" & playlist_name & \
										".m3u failed" \
											with icon caution buttons {"OK"}
								return
							end try
						end try
						tell application "TextEdit"
							activate
							set text of front document to playlist
							set l to POSIX path of (m3ulist as alias)
							set path of front document to l
							save front document
							quit
						end tell
					end if
				end tell

			end repeat

			display dialog "done" buttons {"OK"}
		}
	end

	def windowShouldClose(sender = nil)
		NSApp.stop(self)
		true
	end
end
